package com.ruoyi.analysis.controller;

import com.google.common.collect.Lists;
import com.ruoyi.analysis.constants.HBaseConstant;
import com.ruoyi.analysis.constants.TimeConstant;
import com.ruoyi.analysis.domain.VehicleMessage;
import com.ruoyi.analysis.hbase.service.HBaseService;
import com.ruoyi.collect.domain.CarRecord;
import com.ruoyi.common.core.web.domain.AjaxResult;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

/**
 * 查询历史轨迹
 * @Author： JCC
 * @Date： 2023/8/30 18:44
 * @Description：
 */
@Log4j2
@RestController
@RequestMapping("/analysis")
public class AnalysisController {

    @Autowired
    private HBaseService hBaseService;

    public static SimpleDateFormat targetDay = new SimpleDateFormat("yyyy-MM-dd");

    @PostMapping("queryHistory")
    public AjaxResult queryHistory(@RequestBody CarRecord driverRecord) throws IOException, NoSuchFieldException, InvocationTargetException, IllegalAccessException, ParseException {
        log.info("查询=开始时间：{},结束时间：{}",new Date(Long.parseLong(driverRecord.getStartKey())).toLocaleString(),
                new Date(Long.parseLong(driverRecord.getEndKey())).toLocaleString());
        //根据输入输出时间格式化日期
        String startDay = targetDay.format(new Date(Long.parseLong(driverRecord.getStartKey())));
        String endDay = targetDay.format(new Date(Long.parseLong(driverRecord.getEndKey())));
        //判断开始结束是否同一天
        if (startDay.equals(endDay)){
            //同一天直接通过开始、结束时间戳查询
            List<VehicleMessage> list = hBaseService.scanRange(HBaseConstant.HBASE_TABLE_PREFIX+startDay,
                    driverRecord.getCarVin()+":"+driverRecord.getStartKey(), driverRecord.getCarVin()+":"+driverRecord.getEndKey());
            return AjaxResult.success(list);
        }

        List<VehicleMessage> list = Lists.newArrayList();
        //定义当前日期名称
        String indexDateTime = "";
        //获取开始时间时间戳
        long startTime = Long.parseLong(driverRecord.getStartKey());
        //获取结束时间当日00:00的时间戳
        long endTime = Long.parseLong(driverRecord.getEndKey());
        indexDateTime = HBaseConstant.HBASE_TABLE_PREFIX + targetDay.format(new Date(startTime));
        //将第一天的数据插入集合
        list.addAll(hBaseService.scanRangeByStart(indexDateTime,driverRecord.getCarVin()+":"+startTime));
        long indexTime = targetDay.parse(targetDay.format(startTime+ TimeConstant.DAY_MILLISECOND)).getTime();
        long count = (endTime - indexTime) / TimeConstant.DAY_MILLISECOND;
        for (long l = 0; l < count; l++) {
            String tempTableName = HBaseConstant.HBASE_TABLE_PREFIX + targetDay.format(indexTime);
            list.addAll(hBaseService.scanRangeByStart(tempTableName,driverRecord.getCarVin()+":"+indexTime));
            indexTime += TimeConstant.DAY_MILLISECOND;
        }
        //根据最后一天的时间戳格式化时间
        String format = targetDay.format(new Date(indexTime));
        //获得最后一天的数据添加到集合
        list.addAll(hBaseService.scanRange(HBaseConstant.HBASE_TABLE_PREFIX+format,
                driverRecord.getCarVin()+":"+indexTime,driverRecord.getCarVin()+":"+endTime));
        return AjaxResult.success(list);
    }

}
