package com.ruoyi.analysis.hbase.instance;


import com.ruoyi.analysis.hbase.config.HbaseConfig;
import lombok.extern.log4j.Log4j2;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.hbase.HBaseConfiguration;
import org.apache.hadoop.hbase.client.Admin;
import org.apache.hadoop.hbase.client.Connection;
import org.apache.hadoop.hbase.client.ConnectionFactory;
import org.springframework.context.annotation.Bean;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * HBase连接和管理实例
 */
@Log4j2
@Component
public class HBaseInstance {
    private ExecutorService pool = Executors.newScheduledThreadPool(20); //设置hbase连接池

    /**
     * 初始化连接
     */
    @Bean(name = "hbaseConnection")
    public Connection initConnection(HbaseConfig hbaseConfig){
        try {
            Configuration conf = HBaseConfiguration.create();
            //将hbase配置类中定义的配置加载到连接池中每个连接里
            Map<String, String> confMap = hbaseConfig.getConfMaps();
            for (Map.Entry<String,String> confEntry : confMap.entrySet()) {
                conf.set(confEntry.getKey(), confEntry.getValue());
            }
            return ConnectionFactory.createConnection(conf, pool);
        } catch (IOException e) {
            log.error("连接HBase出错",e);
            return null;
        }
    }

    /**
     * Hbase管理APi
     */
    @Bean(name = "hbaseAdmin")
    public Admin initAdmin(Connection connection){
        try {
            return connection.getAdmin();
        } catch (IOException e) {
            log.error("创建HBase管理API出错",e);
            return null;
        }
    }
}
